@@ -6,8 +6,10 @@ import (
 	"path/filepath"
 	"testing"
 
+	"github.com/containers/image/v5/types"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/exp/slices"
 )
 
 // assertJSONEqualsFixture tests that jsonBytes is structurally equal to fixture,
@@ -29,3 +31,12 @@ func assertJSONEqualsFixture(t *testing.T, jsonBytes []byte, fixture string, ign
 	}
 	assert.Equal(t, fixtureContents, contents)
 }
+
+// layerInfosWithCryptoOperation returns a copy of input where CryptoOperation is set to op
+func layerInfosWithCryptoOperation(input []types.BlobInfo, op types.LayerCrypto) []types.BlobInfo {
+	res := slices.Clone(input)
+	for i := range res {
+		res[i].CryptoOperation = op
+	}
+	return res
+}
